/*
** Apple Macintosh Developer Technical Support
**
** File:		Menu.c
** Written by:	Eric Soldan
**
** Copyright  1990-1991 Apple Computer, Inc.
** All rights reserved.
*/

/* This file manages the application menus. */



/*****************************************************************************/



#include "App.h"			/* Get the application includes/typedefs, etc.	*/
#include "App.Common.h"		/* Get the stuff in common with rez.			*/
#include "App.protos.h"		/* Get the prototypes for application.			*/

#ifndef __DESK__
#include <Desk.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __LISTCONTROL__
#include "ListControl.h"
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __TEXTEDITCONTROL__
#include "TextEditControl.h"
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __UTILITIES__
#include "Utilities.h"
#endif



/*****************************************************************************/



extern Boolean	gQuitApplication;
extern Boolean	gHasAppleEvents;
	/* Above are DTS.Lib..framework globals. */

static Boolean	DoAdjustFileMenu(WindowPtr window);
static Boolean	DoAdjustEditMenu(WindowPtr window);
static Boolean	DoAdjustCommunicateMenu(WindowPtr window);

#define kMinBlockOfRam 0x10000L



/*****************************************************************************/
/*****************************************************************************/



/*  Called by DTS.Lib..framework.  */

/* Enable and disable menus based on the current state.  The user can only
** select enabled menu items.  We set up all the menu items before calling
** MenuSelect or MenuKey, since these are the only times that a menu item can
** be selected.  Note that MenuSelect is also the only time the user will see
** menu items.  This approach to deciding what enable/disable state a menu
** item has the advantage of concentrating all the decision-making in one
** place, as opposed to being spread throughout the application.  Other
** application designs may take a different approach that is just as valid. */

#pragma segment Menu
void	DoAdjustMenus(void)
{
	WindowPtr	window;
	Boolean		redrawMenuBar;

	window = FrontWindow();

	redrawMenuBar  = DoAdjustFileMenu(window);
	redrawMenuBar |= DoAdjustEditMenu(window);
	redrawMenuBar |= DoAdjustCommunicateMenu(window);

	if (redrawMenuBar)
		DrawMenuBar();
}



/*****************************************************************************/



/*  Called by DTS.Lib..framework.  */

/* This is called when an item is chosen from the menu bar (after calling
** MenuSelect or MenuKey).  It performs the right operation for each command.
** It is good to have both the result of MenuSelect and MenuKey go to one
** routine like this to keep everything organized. */

#pragma segment Menu
void	DoMenuCommand(long menuResult)
{
	short		menuID, menuItem, daRefNum, saveMode;
	Str255		daName;
	FileRecHndl	frHndl, newFrHndl;
	WindowPtr	window;
	OSErr		err;

	if (window = FrontWindow())
		frHndl = (FileRecHndl)GetWRefCon(window);
			/* frHndl is valid only if it is one of our windows. */

	menuID = HiWord(menuResult);	/* Use macros for efficiency to get */
	menuItem = LoWord(menuResult);	/* menu item number and menu number. */

	switch (menuID) {

		case mApple:
			switch (menuItem) {
				case iAbout:	/* Bring up alert for About. */
					CenteredAlert(rAboutAlert, nil, (ModalFilterProcPtr)AlertFilter);
					break;
				default:		/* All non-About items in this menu are DAs. */
					GetItem(GetMHandle(mApple), menuItem, daName);
					daRefNum = OpenDeskAcc(daName);
					break;
			}
			break;

		case mFile:
			switch (menuItem) {
				case iNew:
					err = NewDocument(&frHndl, kDocFileType, true);
					if (!err) {
						err = DoNewWindow(frHndl, nil, FrontWindow(), (WindowPtr)-1);
						if (err)
							DisposeDocument(frHndl);
					}
					if (err)
						CenteredAlert(rErrorAlert, nil, (ModalFilterProcPtr)AlertFilter);
					break;
				case iOpen:
					err = OpenDocument(&frHndl, nil, fsRdWrPerm);
					if (!err) {
						err = DoNewWindow(frHndl, nil, FrontWindow(), (WindowPtr)-1);
						if (err)
							DisposeDocument(frHndl);
					}
					if ((err) && (err != userCanceledErr))
						CenteredAlert(rErrorAlert, nil, (ModalFilterProcPtr)AlertFilter);
					break;
				case iClose:
					DisposeOneWindow(window, kClose);
					break;
				case iSave:
				case iSaveAs:
					saveMode = (menuItem == iSave) ? kSave : kSaveAs;
					if ((*frHndl)->fileState.refNum == kInvalRefNum)
						saveMode = kSaveAs;
					err = SaveDocument(frHndl, window, saveMode);
					if ((err) && (err != userCanceledErr))
						CenteredAlert(rErrorAlert, nil, (ModalFilterProcPtr)AlertFilter);
					break;
				case iDuplicate:
					err = DuplicateDocument(frHndl, &newFrHndl);
					if (!err) {
						err = DoNewWindow(newFrHndl, nil, FrontWindow(), (WindowPtr)-1);
						if (err)
							DisposeDocument(newFrHndl);
					}
					if (err)
						CenteredAlert(rErrorAlert, nil, (ModalFilterProcPtr)AlertFilter);
					break;
				case iPageSetup:
					DoSetCursor(&qd.arrow);
					PresentStyleDialog(frHndl);
					break;
				case iPrint:
					DoSetCursor(&qd.arrow);
					err = noErr;
					if (!(*frHndl)->d.doc.fhInfo.printRecValid)
						err = PresentStyleDialog(frHndl);
					if (!err) {
						err = PrintDocument(frHndl, true, true);
						PrintDocument(nil, false, false);
					}
					if ((err) && (err != userCanceledErr))
						CenteredAlert(rErrorAlert, nil, (ModalFilterProcPtr)AlertFilter);
					break;
				case iQuit:
					gQuitApplication = DisposeAllWindows();
					break;
			}
			break;

		case mEdit:			/* Call SystemEdit for DA editing & MultiFinder. */
			if (IsAppWindow(window)) {
				switch (menuItem) {
					case iUndo:
					case iCut:
					case iCopy:
					case iPaste:
					case iClear:
						BeginContent(window);
						switch ((*frHndl)->fileState.sfType) {
							case kDocFileType:
							case kTextFileType:

								if (menuItem == iUndo)
									CTEUndo();
								else
									CTEClipboard(menuItem - iCut + 2);

								if (menuItem != iCopy)
									SetWindowDirty(window);

								break;
#if VH_VERSION
							case kViewHierFileType:
								if (menuItem == iUndo)
									CTEUndo();
								else
									CTEClipboard(menuItem - iCut + 2);
								break;
#endif
						}
						EndContent(window);
						break;
#if VH_VERSION
					case iViewHier:
						err = NewDocument(&frHndl, kViewHierFileType, false);
						if (!err) {
							err = DoNewWindow(frHndl, nil, FrontWindow(), (WindowPtr)-1);
							if (err)
								DisposeDocument(frHndl);
						}
						if (err)
							CenteredAlert(rErrorAlert, nil, (ModalFilterProcPtr)AlertFilter);
						break;
#endif
				}
			}
			else SystemEdit(menuItem - 1);
			break;

		case mCommunicate:
			switch (menuItem) {
				case iConnectToUser:
					DoSetCursor(&qd.arrow);
					if ((*frHndl)->connect.connected)
						SendMessage(frHndl, kDisconnectMssg);
					else
						SendConnect(frHndl, (char *)"\pDTS.Chat");
					break;
				case iSendToUser:
					SendMessage(frHndl, kTextMssg);
					break;

			}
			break;
	}

	HiliteMenu(0);		/* Unhighlight what MenuSelect (or MenuKey) hilited. */
}



/*****************************************************************************/



/* This function either enables or disables a menu item. */

#pragma segment Menu
void	EnableOrDisableItem(MenuHandle menu, short item, Boolean enable)
{
	if (enable)
		EnableItem(menu, item);
	else
		DisableItem(menu, item);
}



/*****************************************************************************/



#pragma segment Menu
static Boolean	DoAdjustFileMenu(WindowPtr window)
{
	MenuHandle	menu;
	FileRecHndl	frHndl;
	short		i, enableItem;

	menu = GetMHandle(mFile);

	if (IsDAWindow(window)) {
		for (i = iNew; i < iQuit; ++i) DisableItem(menu, i);
		EnableItem(menu, iClose);	/* Let DAs do a close from the menu. */
		return(false);
	}

	EnableItem(menu, iNew);					/* Set these for the no-windows state. */
	EnableItem(menu, iOpen);
	if (MaxBlock() < kMinBlockOfRam) {
		if (CompactMem(kMinBlockOfRam) < kMinBlockOfRam) {
			DisableItem(menu, iNew);		/* Running low on RAM. */
			DisableItem(menu, iOpen);
		}
	}

	DisableItem(menu, iClose);
	DisableItem(menu, iSave);
	DisableItem(menu, iSaveAs);
	DisableItem(menu, iDuplicate);
	DisableItem(menu, iPageSetup);
	DisableItem(menu, iPrint);

	if (IsDAWindow(window)) {
		DisableItem(menu, iNew);	/* DAs don't get to do a new. */
		DisableItem(menu, iOpen);	/* DAs don't get to do an open. */
		EnableItem(menu, iClose);	/* Let DAs do a close from the menu. */
	}


	if (window = FrontWindowOfType(kwIsDocument)) {
		EnableItem(menu, iClose);
		frHndl = (FileRecHndl)GetWRefCon(window);
		if ((*frHndl)->fileState.sfType == kDocFileType) {
			enableItem = GetWindowDirty(window);
			if ((*frHndl)->fileState.refNum == kInvalRefNum)
				enableItem = true;
			EnableOrDisableItem(menu, iSave, enableItem);
			EnableItem(menu, iSaveAs);
			EnableItem(menu, iDuplicate);
		}
		EnableItem(menu, iPageSetup);
		EnableItem(menu, iPrint);
	}

	return(false);
}



/*****************************************************************************/



#pragma segment Menu
static Boolean	DoAdjustEditMenu(WindowPtr window)
{
	MenuHandle		menu;
	Boolean			menuEnabled, redrawMenuBar;
	FileRecHndl		frHndl;
	short			i, j;
	static Boolean	editMenuEnabled = true;

	menu = GetMHandle(mEdit);

	if (IsDAWindow(window)) {
		EnableItem(menu, iUndo);
		EnableItem(menu, iCut);
		EnableItem(menu, iCopy);
		EnableItem(menu, iPaste);
		EnableItem(menu, iClear);
#if VH_VERSION
		DisableItem(menu, iViewHier);
#endif
		redrawMenuBar   = !editMenuEnabled;
		editMenuEnabled = true;
		return(redrawMenuBar);
	}

#if VH_VERSION
	j = iViewHier;
#else
	j = iClear;
#endif

	menuEnabled = false;
	for (i = iUndo; i <= j; ++i) DisableItem(menu, i);
	if (IsAppWindow(window)) {
		frHndl = (FileRecHndl)GetWRefCon(window);
		switch ((*frHndl)->fileState.sfType) {
			case kDocFileType:
			case kTextFileType:
				CTEEditMenu(&menuEnabled, mEdit, iUndo, iCut);
				EnableItem(menu, iViewHier);
				menuEnabled = true;
				break;
#if VH_VERSION
			case kViewHierFileType:
				CTEEditMenu(&menuEnabled, mEdit, iUndo, iCut);
				break;
#endif
		}
	}

	redrawMenuBar   = (editMenuEnabled != menuEnabled);
	editMenuEnabled = menuEnabled;
	return(redrawMenuBar);
}



/*****************************************************************************/



#pragma segment Menu
static Boolean	DoAdjustCommunicateMenu(WindowPtr window)
{
	MenuHandle		menu;
	Boolean			menuEnabled, redrawMenuBar;
	FileRecHndl		frHndl;
	static Boolean	communicateMenuEnabled = true;

	menu = GetMHandle(mCommunicate);		/* Start by disabling everything. */
	menuEnabled = false;

	CheckItem(menu, iConnectToUser, false);
	DisableItem(menu, iConnectToUser);
	DisableItem(menu, iSendToUser);

	if (IsAppWindow(window)) {
		frHndl = (FileRecHndl)GetWRefCon(window);
		if (!(*frHndl)->fileState.readOnly) {
			if (gHasAppleEvents) {
				if ((*frHndl)->fileState.sfType == kDocFileType) {
					EnableItem(menu, iConnectToUser);
					menuEnabled = true;
					if ((*frHndl)->connect.connected) {
						DisableItem(menu, iConnectToUser);
						EnableItem(menu, iSendToUser);
					}
					CheckItem(menu, iConnectToUser, (*frHndl)->connect.connected);
				}
			}
		}
	}

	redrawMenuBar = false;
	if (communicateMenuEnabled != menuEnabled) {
		redrawMenuBar = true;
		if (communicateMenuEnabled = menuEnabled)
			EnableItem(menu, 0);
		else
			DisableItem(menu, 0);
	}

	return(redrawMenuBar);
}



