/*
** Apple Macintosh Developer Technical Support
**
** File:		File2.c
** Written by:	Eric Soldan
**
** Copyright  1990-1992 Apple Computer, Inc.
** All rights reserved.
*/

/* This file is where you place various definitions and constant values for
** DTS.Lib..framework's use.  You will also add some code within the two
** functions to handle the various document types. */



/*****************************************************************************/



#include "App.h"			/* Get the application includes/typedefs, etc.	*/
#include "App.Common.h"		/* Get the stuff in common with rez.			*/
#include "App.protos.h"		/* Get the prototypes for the application.		*/

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __TREEOBJ2__
#include "TreeObj2.h"
#endif

#ifndef __UTILITIES__
#include "Utilities.h"
#endif



/* In this file, we first set some global values.  This allows the application and
** DTS.Lib..framework to "know" what is expected for certain default actions. */


OSType		gDocCreator = kDocCreator;	/* So DTS.Lib..framework knows who's boss. */

short		gTypeListLen = 1;
SFTypeList	gTypeList = {kDocFileType};
	/* Here we declare the various document types that DTS.Draw can support.
	8* These definitions are to inform DTS.Lib..framework what documents can be opened. */



/* Some DTS.Lib..framework gTypeList usage notes:
**
** 1)  Framework uses gTypeList[0] for the default document, if there is one.
** 2)  NewDocument() is passed a document type.  It searches gTypeList for a match.
**     The index at which the match is found (+1) is used as the string number in the
**     STR# resource rDefaultTitles.  If there aren't enough strings in the STR#
**     resource, then the last string is used.
** 3)  The gTypeList is used for the StandardFile calls to determine which files
**     can be selected. */



typedef struct DocFileTypeRec {		/* This is used only to determine the size of the document */
	FileStateRec	fileState;		/* structure.  We can't just add the three components, as  */
	ConnectRec		connect;		/* it is unclear how much padding any particular compiler  */
	TheDoc			doc;			/* will place on the end of each.						   */
} DocFileTypeRec;



/* Below are the TreeObj procedure pointers for the various kinds of objects we use in this
** application.  The first 16 are reserved for the framework.  Our application-specific
** objects start at 16. */

TreeObjProcPtr	gTreeObjMethods[kNumTreeObjs] = {nil,
/* 1  */										 TRootObj,
/* 2  */										 TUndoObj,
/* 3  */										 TUndoTaskObj,
/* 4  */										 TUndoPartObj,
/* 5  */										 nil,
/* 6  */										 nil,
/* 7  */										 nil,
/* 8  */										 nil,
/* 9  */										 nil,
/* 10 */										 nil,
/* 11 */										 nil,
/* 12 */										 nil,
/* 13 */										 nil,
/* 14 */										 nil,
/* 15 */										 nil,
/* 16		Start of app-specific procs. */		 TRectObj,
/* 17 */										 TRRectObj,
/* 18 */										 TOvalObj,
/* 19 */										 TPieObj,
/* 20 */										 nil,		/* Reserve space for other   */
/* 21 */										 nil,		/* objects that may be added */
/* 22 */										 nil,		/* to the tool palette.      */
/* 23 */										 nil,		/* The group object is never */
/* 24 */										 nil,		/* going to be in the tool   */
/* 25 */										 nil,		/* palette, so put it far    */
/* 26 */										 nil,		/* enough away to leave room */
/* 27 */										 nil,		/* for other objects.        */
/* 28 */										 nil,
/* 29 */										 nil,
/* 30 */										 nil,
/* 31 */										 nil,
/* 32 */										 nil,
/* 33 */										 nil,
/* 34 */										 nil,
/* 35 */										 nil,
/* 36 */										 nil,
/* 37 */										 nil,
/* 38 */										 nil,
/* 39 */										 nil,
/* 40 */										 nil,
/* 41 */										 nil,
/* 42 */										 nil,
/* 43 */										 nil,
/* 44 */										 nil,
/* 45 */										 nil,
/* 46 */										 nil,
/* 47 */										 nil,
/* 48 */										 nil,
/* 49 */										 nil,
/* 50 */										 nil,
/* 51 */										 nil,
/* 52 */										 nil,
/* 53 */										 nil,
/* 54 */										 nil,
/* 55 */										 nil,
/* 56 */										 nil,
/* 57 */										 nil,
/* 58 */										 nil,
/* 59 */										 nil,
/* 60 */										 nil,
/* 61 */										 nil,
/* 62 */										 nil,
/* 63 */										 nil,
/* 64 */										 nil,
/* 65 */										 TGroupObj};



/* The framework needs to know the minimum object sizes.  This table is used by the
** framework to make sure that the object is created at least minimally. */

long			gMinTreeObjSize[kNumTreeObjs] = {0,
/* 1  */										 sizeof(RootObj),
/* 2  */										 sizeof(UndoObj),
/* 3  */										 sizeof(UndoTaskObj),
/* 4  */										 sizeof(UndoPartObj),
/* 5  */										 0,
/* 6  */										 0,
/* 7  */										 0,
/* 8  */										 0,
/* 9  */										 0,
/* 10 */										 0,
/* 11 */										 0,
/* 12 */										 0,
/* 13 */										 0,
/* 14 */										 0,
/* 15 */										 0,
/* 16		Start of app-specific sizes. */		 sizeof(RectObj),
/* 17 */										 sizeof(RRectObj),
/* 18 */										 sizeof(OvalObj),
/* 19 */										 sizeof(PieObj),
/* 20 */										 0,
/* 21 */										 0,
/* 22 */										 0,
/* 23 */										 0,
/* 24 */										 0,
/* 25 */										 0,
/* 26 */										 0,
/* 27 */										 0,
/* 28 */										 0,
/* 29 */										 0,
/* 30 */										 0,
/* 31 */										 0,
/* 32 */										 0,
/* 33 */										 0,
/* 34 */										 0,
/* 35 */										 0,
/* 36 */										 0,
/* 37 */										 0,
/* 38 */										 0,
/* 39 */										 0,
/* 40 */										 0,
/* 41 */										 0,
/* 42 */										 0,
/* 43 */										 0,
/* 44 */										 0,
/* 45 */										 0,
/* 46 */										 0,
/* 47 */										 0,
/* 48 */										 0,
/* 49 */										 0,
/* 50 */										 0,
/* 51 */										 0,
/* 52 */										 0,
/* 53 */										 0,
/* 54 */										 0,
/* 55 */										 0,
/* 56 */										 0,
/* 57 */										 0,
/* 58 */										 0,
/* 59 */										 0,
/* 60 */										 0,
/* 61 */										 0,
/* 62 */										 0,
/* 63 */										 0,
/* 64 */										 0,
/* 65 */										 sizeof(GroupObj)};



/*****************************************************************************/
/*****************************************************************************/



/*  Called by DTS.Lib..framework.  */

/* Do any additional document initialization here.  All fields not specifically set
** are already initialized to 0. */

#pragma segment File
OSErr	InitDocument(FileRecHndl frHndl)
{
	OSErr	err;

	err = noErr;

	switch ((*frHndl)->fileState.sfType) {
		case kDocFileType:
			err = DefaultInitDocument(frHndl, kVersion, kMaxNumUndos, kNumSaveUndos);
			if (!err) {
				(*frHndl)->d.doc.fhInfo.hDocSize = (7 * 72);
				(*frHndl)->d.doc.fhInfo.vDocSize = (10 * 72);
			}
			break;
		case kClipboardFileType:
			ClipboardInitDocument(frHndl);
			break;
		case kToolFileType:
			return(ToolInitDocument(frHndl));
			break;
#if VH_VERSION
		case kViewHierFileType:
			return(VHInitDocument(frHndl));
			break;
#endif
	}

	return(err);
}



/*****************************************************************************/



/*  Called by DTS.Lib..framework.  */

/* Return the initial size of the primary document handle, based on the OSType. */

#pragma segment File
long	InitDocumentSize(OSType sftype)
{
	switch (sftype) {
		case kDocFileType:
		case kClipboardFileType:
		case kToolFileType:
			return(sizeof(DocFileTypeRec));
			break;
#if VH_VERSION
		case kViewHierFileType:
			return(VHFileTypeSize());
			break;
#endif
	}
}



