#ifndef __DTSLib__
#define __DTSLib__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __FILES__
#include <Files.h>
#endif

#ifndef __MOVIES__
#include <Movies.h>
#endif

#ifndef __PRINTING__
#include <Printing.h>
#endif

#ifndef __TREEOBJ__
#include "TreeObj.h"
#endif

#ifndef __WINDOWS__
#include <Windows.h>
#endif

/********/

typedef WindowPtr	(*GetDocWindowProcPtr)(short id, Ptr storage, Boolean vis, WindowPtr relWindow,
										   WindowPtr behind, Boolean inColor, Rect sizeInfo, long refCon);
typedef void		(*CalcFrameRgnProcPtr)(FileRecHndl frHndl, WindowPtr window, RgnHandle rgn);
typedef void		(*ContentClickProcPtr)(WindowPtr window, EventRecord *event, Boolean firstClick);
typedef Boolean		(*ContentKeyProcPtr)(WindowPtr window, EventRecord *event, Boolean *passThrough);
typedef void		(*DrawFrameProcPtr)(FileRecHndl frHndl, WindowPtr window);
typedef OSErr		(*FreeDocumentProcPtr)(FileRecHndl frHndl);
typedef OSErr		(*FreeWindowProcPtr)(FileRecHndl frHndl, WindowPtr window);
typedef OSErr		(*ImageProcPtr)(FileRecHndl frHndl);
typedef OSErr		(*InitContentProcPtr)(FileRecHndl frHndl, WindowPtr window);
typedef OSErr		(*ReadDocumentProcPtr)(FileRecHndl frHndl);
typedef OSErr		(*ReadDocumentHeaderProcPtr)(FileRecHndl frHndl);
typedef void		(*ResizeContentProcPtr)(WindowPtr window, short oldh, short oldv);
typedef void		(*ScrollFrameProcPtr)(FileRecHndl frHndl, WindowPtr window, long dh, long dv);
typedef void		(*UndoFixupProcPtr)(FileRecHndl frHndl, Point contOrg, Boolean afterUndo);
typedef Boolean		(*WindowCursorProcPtr)(FileRecHndl frHndl, WindowPtr window, Point globalPt);
typedef OSErr		(*WriteDocumentProcPtr)(FileRecHndl frHndl);
typedef OSErr		(*WriteDocumentHeaderProcPtr)(FileRecHndl frHndl);

typedef long		(*DocScrollBarProc)(FileRecHndl frHndl, ControlHandle ctl, short message, long val);
#define kscrollHAdjust        0
#define kscrollVAdjust        1
#define kscrollGetHOrigin     2
#define kscrollGetVOrigin     3
#define kscrollGetHLongOrigin 4
#define kscrollGetVLongOrigin 5


typedef struct {
	OSType						sfType;
	Boolean						defaultDoc;
	Movie						movie;
	short						movieResID;
	short						movieFlags;
	Boolean						movieDataRefWasChanged;
	Boolean						docDirty;
	long						modNum;
	long						modTick;
	Boolean						readOnly;
	short						refNum;
	short						resRefNum;
	FSSpec						fss;
	short						windowID;
	WindowPtr					window;
	GetDocWindowProcPtr			getDocWindow;
	CalcFrameRgnProcPtr			calcFrameRgnProc;
	ContentClickProcPtr			contentClickProc;
	ContentKeyProcPtr			contentKeyProc;
	DrawFrameProcPtr			drawFrameProc;
	FreeDocumentProcPtr			freeDocumentProc;
	FreeWindowProcPtr			freeWindowProc;
	ImageProcPtr				imageProc;
	InitContentProcPtr			initContentProc;
	ReadDocumentProcPtr			readDocumentProc;
	ReadDocumentHeaderProcPtr	readDocumentHeaderProc;
	ResizeContentProcPtr		resizeContentProc;
	ScrollFrameProcPtr			scrollFrameProc;
	UndoFixupProcPtr			undoFixupProc;
	WindowCursorProcPtr			windowCursorProc;
	WriteDocumentProcPtr		writeDocumentProc;
	WriteDocumentHeaderProcPtr	writeDocumentHeaderProc;
	short						attributes;			/* Here down is window content information. */
	Rect						windowSizeBounds;
	ControlHandle				hScroll;
	ControlHandle				vScroll;
	short						hScrollIndent;
	short						vScrollIndent;
	short						leftSidebar;
	short						topSidebar;
	short						hArrowVal;
	short						vArrowVal;
	short						hPageVal;
	short						vPageVal;
} FileStateRec;

typedef struct {
	long			windowTag[2];		/* Used to match up windows.					*/
	short			endSendInfo;		/* Above is send info.							*/

	Boolean			connected;			/* Flag showing we are connected.				*/
	AEAddressDesc	remoteLoc;			/* AppleEvents address of remote user.			*/
	Str32			remoteName;			/* Name of user connected to.					*/
	Str32			remoteZone;			/* Zone of user connected to.					*/
	Str32			remoteMachine;		/* Machine name of user connected to.			*/
	short			endLocalInfo;		/* Above info is for one machine only.			*/
} ConnectRec;

typedef struct {
	short		version;			/* The file format version.									*/
	Boolean		printRecValid;		/* True if print record has been created.					*/
	TPrint		print;				/* Print record for file.									*/
	Rect		structureRect;		/* Remember where the window was when saved.				*/
	Rect		contentRect;		/* Remember where the window was when saved.				*/
	Rect		stdState;			/* This rect and below rect used for saving					*/
	Rect		userState;			/* zoom information for window.								*/
	long		hDocSize;			/* hDocSize and vDocSize have to be saved with the document */
	long		vDocSize;			/* so that the window can be created the correct size.      */
	short		endDocHeaderInfo;	/* End version, print, and window information.				*/
} DocHeaderInfo;

struct AEHandler{
	AEEventClass	theEventClass;
	AEEventID		theEventID;
	ProcPtr			theHandler;
};
typedef struct AEHandler AEHandler;


/********/

#define mDerefWStateData(window) (*(WStateDataHandle)(((WindowPeek)window)->dataHandle))

#define kwGrowIcon			1
#define kwHScroll			2
#define kwHScrollLessGrow	6
#define kwVScroll			8
#define kwVScrollLessGrow	24
#define kwVisible			32
#define kwOpenAtOldLoc		64
#define kwDoFirstClick		128
#define kwHideOnClose		256
#define kwIsDocument		0
#define kwIsPalette			512
#define kwIsModalDialog		1024

#define kwNoChange			-32767
#define kwBotScroll			32767

#define kMinWindowWidth  200
#define kMinWindowHeight 200
#define kMaxWindowWidth  16384
#define kMaxWindowHeight 16384

#define kwVHAppWindow		(kwVisible)

#define kCustomEventClass  'CUST'
#define keySFType          'KSFT'
#define keyFSS             'KFSS'
#define keyPascal          'PSTR'
#define keyAppConnect      'KCNT'
#define keyWindowTag	   'KWND'

#define typeAppConnect     'KCNT'
#define typeDoubleLong     'DBLL'
#define typePascal         'PSTR'

#define keyReplyErr        'errn'

#define kWrongVersion	1

#define kInvalRefNum	0
#define kInvalVRefNum	0

#define kSaveYes		1
#define kSaveNo			3
#define kSaveCanceled	4

#define kOpenYes		1
#define kOpenNo			3

#define kOpenMovie ((FSSpecPtr)-1)

#endif __DTSLib__
