#------------------------------------------------------------------------------
#
#	Apple Macintosh Developer Technical Support
#
#	MultiFinder-Aware Sample Application
#
#	Program:	Kibitz
#	File:		Kibitz.make	-	Make Source
#
#	Copyright  1988-1992 Apple Computer, Inc.
#	All rights reserved.
#
#------------------------------------------------------------------------------

AppName			=	'Kibitz'
Signature		=	'KBTZ'

DTS.Lib.folder	=	"::DTS.Lib:"
DTS.Lib.hdrs	=	"::DTS.Lib:DTS.Lib.headers:"
projsrc			=	:
obj				= 	:OBJECT:

#------------------------------------------------------------------------------
# Options for our compilers:
#	-sym on: tells the compilers and linker to emit symbol information for
#		a source level debugger, such as SADE.
#	-i {DTS.Lib.hdrs}: means to look for any #include files in the specified
#		directory, as well as the normal set.
#	-r: tells the C compiler to require function prototypes.
#	-mbg off: tells the compilers to not emit low-level debugger names. This
#		saves on file space, but you may wish to remove this option if you
#		need to debug with something like Macsbug.
#	-rd: for Rez means to suppress warnings for redeclared types (we redeclare
#		'RECT' because its not included in MPW 3.0).
#	-append: means to add the resources to the target file, rather than
#		deleting all the ones that are there first.
#	-d Signature...: is a way of passing our application's signature to Rez.
#		With this mechanism, we can define our signature here, and export
#		it to Rez, so that we don't have to declare it there, too.
#	-sn STDCLIB=Main: puts all the routines that would normally go into the
#		STDCLIB segment into the Main segment. This is done so that when we
#		call upon any low-level utilities, we don't potentially move memory
#		by loading in a segment.
#------------------------------------------------------------------------------

SymOptions		=	#-sym on					# turn this on to debug with SADE
IncludesFolders	=	-i {DTS.Lib.hdrs}

COptions		=	{IncludesFolders} {SymOptions} -r -mbg on
RezOptions		=	{IncludesFolders} -rd -append -d Signature="{Signature}" -d AppName='"Kibitz"'
LinkOptions		=	{SymOptions} {SegmentMappings} -msg nodup
SegmentMappings	=	-sn INTENV=Main 
					-sn PASLIB=Main 
					-sn STDCLIB=Main 
					-sn SANELIB=Main 
					-sn StringUtils=Main 
					-sn UtilMain=Main

#------------------------------------------------------------------------------
# These are modified default build rules.  This is necessary to take into
# account differences between MPW 3.1 and 3.2
#------------------------------------------------------------------------------
{obj}				{projsrc}

{obj}.p.o			{projsrc}.p
	{Pascal} {POptions} {PAltOptions} {DepDir}{Default}.p -o {TargDir}{Default}.p.o

{obj}.c.o			{projsrc}.c
	{C} {COptions} {CAltOptions} {DepDir}{Default}.c -o {TargDir}{Default}.c.o

#------------------------------------------------------------------------------
# These are the objects that we want to link with. If any one of these
# changes, then we invoke the Link command.
#------------------------------------------------------------------------------
AppObjects		=	
					{obj}aechess.c.o 
					{obj}appleevents.c.o 
					{obj}boardslider.c.o 
					{obj}chess.c.o 
					{obj}cmqueenmate.c.o 
					{obj}cmrookmate.c.o 
					{obj}config.c.o 
					{obj}docursor.c.o 
					{obj}doevent.c.o 
					{obj}eventloop.c.o 
					{obj}file.c.o 
					{obj}gotomove.c.o 
					{obj}help.c.o 
					{obj}idletasks.c.o 
					{obj}init.c.o 
					{obj}kibitzwindow.c.o 
					{obj}menu.c.o 
					{obj}notation.c.o 
					{obj}offscreen.c.o 
					{obj}print.c.o 
					{obj}saveboardimage.c.o 
					{obj}setup.c.o 
					{obj}sound.c.o 
					{obj}start.c.o 
					{obj}window.c.o

#------------------------------------------------------------------------------
# These help define the libraries that we want to link with. {AppObjects} holds
# the names of the application units we want to link together. {CLibs} holds
# the libraries we want to link with under MPW 3.0 or MPW 3.1. Under MPW 3.2
# and later, CInterface.o and CRuntime.o are merged with Interface.o and
# Runtime.o. So, under 3.2 and later, we link with the files in {CLibs32}
# instead.
#------------------------------------------------------------------------------
CLibs32			=	
					"{Libraries}Runtime.o" 
					"{DTS.Lib.folder}DTS.Lib" 
#					"{CLibraries}StdCLib.o" 
					"{Libraries}Interface.o"

#------------------------------------------------------------------------------
# Dependencies for the individual components. These will invoke the
# default build rules listed in Chapter 9 of the MPW 3.0 manual.
# You may wish to reduce the number of dependencies.  Two dependencies
# you may wish to remove are this makefile and the Kibitz.protos file.
# For the Kibitz.protos file, if you add a function to the list of
# functions in Kibitz.protos, you will cause all the source files to be
# recompiled.  This may be more than you want to wait for each time you
# add a function to your application.  On the other hand, if you do not
# include this in the dependencies, and you change the parameters for a
# function, and make the respective change to Kibitz.protos, any files
# that reference that function will not be recompiled.  If these files are
# recompiled, the prototype checking will catch any cases where you did
# not change the way the altered function was called.  <<You choose>>
#------------------------------------------------------------------------------

{AppObjects}		#{AppName}.make 
#					Kibitz.h 
#					Kibitz.protos 
					{DTS.Lib.folder}DTS.Lib

#------------------------------------------------------------------------------
# This is a dummy dependency rule.  This will always be executed.  This dummy
# rule must be the first for {AppName} so that it will be executed first.
# This is necessary to make evaluations that are beyond the scope of
# Make.  These evaluations will be performed by the Shell at execution time,
# and they must execute first because compile and link command lines depend
# on variables set up by these evaluations.  This has the unfortunate side
# effect that Make will always consider {AppName} to be out of date.  It will
# always, at a minimum, execute the commands for the target ShellForce.
#------------------------------------------------------------------------------

{AppName}			 ShellForce

# With the above rule, {AppName} will always be out of date with respect to
# the non-existent file ShellForce.  This will force the following commands to
# be executed.
ShellForce			
	BEGIN
		IF "{ShellVersion}" == ""
			( EVALUATE "`Version`" =~ /MPW Shell ([0-9]+(.[ab0-9]+)+)1/ )  Dev:Null
			SET ShellVersion "{1}"
		END
		IF "{ShellVersion}" =~ /3.[01]/
			SET PAltOptions "-d MPW32=FALSE"
			SET CAltOptions "-d MPW31"
			SET PSysObjects "`QUOTE {PLibs}`"
			SET CSysObjects "`QUOTE {CLibs}`"
		ELSE
			SET PAltOptions "-d MPW32=TRUE"
			SET CAltOptions "-d MPW32"
			SET PSysObjects "`QUOTE {PLibs32}`"
			SET CSysObjects "`QUOTE {CLibs32}`"
		END
	END  Dev:Null # Output to bit bucket so we dont see above calculations

#------------------------------------------------------------------------------
# Build rule that links our application together. If any of our objects 
# changes, or this makefile changes, then we relink.  The dummy prerequisite
# ShellForce must come before any other prerequisites for {AppName}
#------------------------------------------------------------------------------

{AppName}			 {AppObjects}
	Link {LinkOptions} -o {Targ} {AppObjects} {CSysObjects}
	SetFile {Targ} -t APPL -c {Signature} -a B

#------------------------------------------------------------------------------
# Build rule that creates our resources and adds them to the application
#------------------------------------------------------------------------------

{AppName}			{AppName}.make 
					Kibitz.r 
					Kibitz.h
	Rez {RezOptions} Kibitz.r -o {Targ}
