/*
**	Apple Macintosh Developer Technical Support
**
**	A collection of useful high-level File Manager routines.
**
**	by Jim Luther, Apple Developer Technical Support
**
**	File:		MoreFilesExtras.c
**
**	Copyright  1992-1993 Apple Computer, Inc.
**	All rights reserved.
*/

#ifndef __MOREFILES__
#include "MoreFilesExtras.h"
#endif

/*****************************************************************************/

/* static prototypes */

static	Ptr	GetSearchBuffer(long *buffSize);

/*****************************************************************************/

/* A utility routine called by NameFileSearch and CreatorTypeFileSearch to
** allocate a optimization buffer. */

static	Ptr	GetSearchBuffer(long *buffSize)
{
	Ptr	tempPtr;
	
	*buffSize = 0x00004000;	/* start by trying for a 16K buffer */
	while (*buffSize) {		/* and keep trying until *buffSize is zero */
		tempPtr = NewPtr(*buffSize);
		if (tempPtr)
			break;	/* got it */
		else
			*buffSize -= 0x00000400;	/* didn't get it, subtract 1K and try again */
	}
	return (tempPtr);
}

/*****************************************************************************/

pascal	OSErr	NameFileSearch(StringPtr volName,
							   short vRefNum,
							   const Str255 fileName,
							   FSSpecPtr matches,
							   long reqMatchCount,
							   long *actMatchCount,
							   Boolean newSearch,
							   Boolean partial)
{
	CInfoPBRec		searchInfo1, searchInfo2;
	HParamBlockRec	pb;
	OSErr			error;
	static CatPositionRec catPosition;
	static short	lastVRefNum = 0;
	
	/* get the real volume reference number */
	error = DetermineVRefNum(volName, vRefNum, &vRefNum);
	if (error != noErr)
		return (error);
	
	pb.csParam.ioNamePtr = nil;
	pb.csParam.ioVRefNum = vRefNum;
	pb.csParam.ioMatchPtr = matches;
	pb.csParam.ioReqMatchCount = reqMatchCount;
	pb.csParam.ioSearchBits = (partial) ?	/* tell CatSearch what we're looking for: */
		(fsSBPartialName + fsSBFlAttrib) : 	/* partial name file matches or */
		(fsSBFullName + fsSBFlAttrib);		/* full name file matches */
	pb.csParam.ioSearchInfo1 = &searchInfo1;
	pb.csParam.ioSearchInfo2 = &searchInfo2;
	pb.csParam.ioSearchTime = 0;
	if ((newSearch) ||				/* If caller specified new search */
		(lastVRefNum != vRefNum))	/* or if last search was to another volume, */
		catPosition.initialize = 0;	/* then search from beginning of catalog */
	pb.csParam.ioCatPosition = catPosition;
	pb.csParam.ioOptBuffer = GetSearchBuffer(&pb.csParam.ioOptBufSize);

	/* search for fileName */
	searchInfo1.hFileInfo.ioNamePtr = (StringPtr)fileName;
	searchInfo2.hFileInfo.ioNamePtr = nil;
	
	/* only match files (not directories) */
	searchInfo1.hFileInfo.ioFlAttrib = 0x00;
	searchInfo2.hFileInfo.ioFlAttrib = 0x10;

	error = PBCatSearchSync((CSParamPtr)&pb);
	
	if ((error == noErr) ||							/* If no errors or the end of catalog was */
		(error == eofErr))							/* found, then the call was successful so */
		*actMatchCount = pb.csParam.ioActMatchCount;	/* return the match count */
	else
		*actMatchCount = 0;							/* else no matches found */
	
	if ((error == noErr) ||							/* If no errors */
		(error == catChangedErr))					/* or there was a change in the catalog */
	{
		catPosition = pb.csParam.ioCatPosition;
		lastVRefNum = vRefNum;
			/* we can probably start the next search where we stopped this time */
	}
	else
		catPosition.initialize = 0;
			/* start the next search from beginning of catalog */
	
	if (pb.csParam.ioOptBuffer)
		DisposPtr(pb.csParam.ioOptBuffer);
		
	return (error);
}

/*****************************************************************************/

pascal	OSErr	CreatorTypeFileSearch(StringPtr volName,
									  short vRefNum,
									  OSType creator,
									  OSType fileType,
									  FSSpecPtr matches,
									  long reqMatchCount,
									  long *actMatchCount,
									  Boolean newSearch)
{
	CInfoPBRec		searchInfo1, searchInfo2;
	HParamBlockRec	pb;
	OSErr			error;
	static CatPositionRec catPosition;
	static short	lastVRefNum = 0;
	
	/* get the real volume reference number */
	error = DetermineVRefNum(volName, vRefNum, &vRefNum);
	if (error != noErr)
		return (error);
	
	pb.csParam.ioNamePtr = nil;
	pb.csParam.ioVRefNum = vRefNum;
	pb.csParam.ioMatchPtr = matches;
	pb.csParam.ioReqMatchCount = reqMatchCount;
	pb.csParam.ioSearchBits = fsSBFlAttrib + fsSBFlFndrInfo;	/* Looking for finder info file matches */
	pb.csParam.ioSearchInfo1 = &searchInfo1;
	pb.csParam.ioSearchInfo2 = &searchInfo2;
	pb.csParam.ioSearchTime = 0;
	if ((newSearch) ||				/* If caller specified new search */
		(lastVRefNum != vRefNum))	/* or if last search was to another volume, */
		catPosition.initialize = 0;	/* then search from beginning of catalog */
	pb.csParam.ioCatPosition = catPosition;
	pb.csParam.ioOptBuffer = GetSearchBuffer(&pb.csParam.ioOptBufSize);

	/* no fileName */
	searchInfo1.hFileInfo.ioNamePtr = nil;
	searchInfo2.hFileInfo.ioNamePtr = nil;
	
	/* only match files (not directories) */
	searchInfo1.hFileInfo.ioFlAttrib = 0x00;
	searchInfo2.hFileInfo.ioFlAttrib = 0x10;
	
	/* search for creator; if creator = 0x00000000, ignore creator */
	searchInfo1.hFileInfo.ioFlFndrInfo.fdCreator = creator;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdCreator = (creator == (OSType)0x00000000) ?
		(OSType)0x00000000 :
		(OSType)0xffffffff;
	
	/* search for fileType; if fileType = 0x00000000, ignore fileType */
	searchInfo1.hFileInfo.ioFlFndrInfo.fdType = fileType;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdType = (fileType == (OSType)0x00000000) ?
		(OSType)0x00000000 :
		(OSType)0xffffffff;
	
	/* zero all other FInfo fields */
	searchInfo1.hFileInfo.ioFlFndrInfo.fdFlags = 0;
	searchInfo1.hFileInfo.ioFlFndrInfo.fdLocation.v = 0;
	searchInfo1.hFileInfo.ioFlFndrInfo.fdLocation.h = 0;
	searchInfo1.hFileInfo.ioFlFndrInfo.fdFldr = 0;
	
	searchInfo2.hFileInfo.ioFlFndrInfo.fdFlags = 0;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdLocation.v = 0;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdLocation.h = 0;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdFldr = 0;

	error = PBCatSearchSync((CSParamPtr)&pb);
	
	if ((error == noErr) ||							/* If no errors or the end of catalog was */
		(error == eofErr))							/* found, then the call was successful so */
		*actMatchCount = pb.csParam.ioActMatchCount;	/* return the match count */
	else
		*actMatchCount = 0;							/* else no matches found */
	
	if ((error == noErr) ||							/* If no errors */
		(error == catChangedErr))					/* or there was a change in the catalog */
	{
		catPosition = pb.csParam.ioCatPosition;
		lastVRefNum = vRefNum;
			/* we can probably start the next search where we stopped this time */
	}
	else
		catPosition.initialize = 0;
			/* start the next search from beginning of catalog */
	
	if (pb.csParam.ioOptBuffer)
		DisposPtr(pb.csParam.ioOptBuffer);
		
	return (error);
}

/*****************************************************************************/

pascal	OSErr	DetermineVRefNum(StringPtr pathname,
								 short vRefNum,
								 short *realVRefNum)
{
	HParamBlockRec pb;
	Str255 tempPathname;
	OSErr error;

	pb.volumeParam.ioVRefNum = vRefNum;
	if (pathname == nil) {
		pb.volumeParam.ioNamePtr = nil;
		pb.volumeParam.ioVolIndex = 0;		/* use ioVRefNum only */
	}
	else {
		pcpy((StringPtr)tempPathname, pathname);	/* make a copy of the string and */
		pb.volumeParam.ioNamePtr = (StringPtr)tempPathname;	/* use the copy so original isn't trashed */
		pb.volumeParam.ioVolIndex = -1;	/* use ioNamePtr/ioVRefNum combination */
	}
	error = PBHGetVInfoSync(&pb);
	*realVRefNum = pb.volumeParam.ioVRefNum;
	return (error);
}

/*****************************************************************************/

pascal	OSErr	OnLine(FSSpecPtr volumes,
					   short reqVolCount,
					   short *actVolCount,
					   short *volIndex)
{
	HParamBlockRec pb;
	OSErr error = noErr;
	FSSpec *endVolArray = volumes + reqVolCount;

	*actVolCount = 0;
	for (; (volumes < endVolArray) && (error == noErr); ++volumes)
	{
		pb.volumeParam.ioNamePtr = (StringPtr) & volumes->name;
		pb.volumeParam.ioVolIndex = *volIndex;
		error = PBHGetVInfoSync(&pb);
		if (error == noErr)
		{
			volumes->parID = fsRtParID;		/* the root directory's parent is 1 */
			volumes->vRefNum = pb.volumeParam.ioVRefNum;
			++*volIndex;
			++*actVolCount;
		}
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	GetDirID(short vRefNum,
						 long dirID,
						 StringPtr name,
						 long *theDirID,
						 Boolean *isDirectory)
{
	CInfoPBRec pb;
	OSErr error;

	pb.hFileInfo.ioNamePtr = name;
	pb.hFileInfo.ioVRefNum = vRefNum;
	pb.hFileInfo.ioDirID = dirID;
	pb.hFileInfo.ioFDirIndex = 0;	/* use ioNamePtr and ioDirID */
	error = PBGetCatInfoSync(&pb);
	*theDirID = pb.hFileInfo.ioDirID;
	*isDirectory = (pb.hFileInfo.ioFlAttrib & 0x10) != 0;
	return (error);
}

/*****************************************************************************/

pascal	OSErr	DirIDFromFSSpec(const FSSpec *spec,
								long *theDirID,
								Boolean *isDirectory)
{
	return (GetDirID(spec->vRefNum, spec->parID, (StringPtr)spec->name,
			theDirID, isDirectory));
}

/*****************************************************************************/

pascal	OSErr	GetDirName(short vRefNum,
						   long dirID,
						   StringPtr name)
{
	CInfoPBRec pb;
	OSErr error;

	pb.hFileInfo.ioNamePtr = name;
	pb.hFileInfo.ioVRefNum = vRefNum;
	pb.hFileInfo.ioDirID = dirID;
	pb.hFileInfo.ioFDirIndex = -1;	/* get information about ioDirID */
	return (PBGetCatInfoSync(&pb));
}

/*****************************************************************************/

pascal	OSErr	GetParentID(short vRefNum,
							long dirID,
							StringPtr name,
							long *parID)
{
	CInfoPBRec pb;
	OSErr error;

	pb.hFileInfo.ioNamePtr = name;
	pb.hFileInfo.ioVRefNum = vRefNum;
	pb.hFileInfo.ioDirID = dirID;
	pb.hFileInfo.ioFDirIndex = 0;				/* use ioNamePtr and ioDirID */
	error = PBGetCatInfoSync(&pb);
	*parID = pb.hFileInfo.ioFlParID;
	return (error);
}

/*****************************************************************************/

pascal	void	GetFilenameFromPathname(const Str255 pathname,
										Str255 filename)
{
	short index;

	index = pathname[0];
	if ((pathname[index] == ':') || (!index)) {
		filename[0] = 0;
		return;
	}
	
	while ((pathname[index] != ':') && (index != 0))
		--index;
		
	if (!index) 
		filename[0] = 0;
	else {
		filename[0] = pathname[0] - index;
		BlockMove(&pathname[index+1], &filename[1], pathname[0] - index);
	}
}

/*****************************************************************************/

pascal	OSErr	GetDirItems(short vRefNum,
							long dirID,
							StringPtr name,
							Boolean getFiles,
							Boolean getDirectories,
							FSSpecPtr items,
							short reqItemCount,
							short *actItemCount,
							short *itemIndex) /* start with 1, then use what's returned */
{
	CInfoPBRec pb;
	OSErr error = noErr;
	long theDirID;
	Boolean isDirectory;
	FSSpec *endItemsArray = items + reqItemCount;

	/* NOTE: If I could be sure that the caller passed a real vRefNum and real directory */
	/* to this routine, I could rip out calls to DetermineVRefNum and GetDirID and this */
	/* routine would be much faster because of the overhead of DetermineVRefNum and */
	/* GetDirID and because GetDirID blows away the directory index hint the Macintosh */
	/* file system keeps for indexed calls. I can't be sure, so for maximum throughput, */
	/* pass a big array of FSSpecs so you can get the directory's contents with few calls */
	/* to this routine. */
	
	/* get the real volume reference number */
	error = DetermineVRefNum(name, vRefNum, &pb.hFileInfo.ioVRefNum);
	if (error != noErr)
		return (error);
	
	/* and the real directory ID of this directory (and make sure it IS a directory */
	error = GetDirID(vRefNum, dirID, name, &theDirID, &isDirectory);
	if (error != noErr)
		return (error);
	else if (!isDirectory)
		return (dirNFErr);


	*actItemCount = 0;
	for (; (items < endItemsArray) && (error == noErr); )
	{
		pb.hFileInfo.ioNamePtr = (StringPtr) &items->name;
		pb.hFileInfo.ioDirID = theDirID;
		pb.hFileInfo.ioFDirIndex = *itemIndex;
		error = PBGetCatInfoSync(&pb);
		if (error == noErr)
		{
			items->parID = pb.hFileInfo.ioFlParID;	/* return item's parID */
			items->vRefNum = pb.hFileInfo.ioVRefNum;	/* return item's vRefNum */
			++*itemIndex;	/* prepare to get next item in directory */
			
			if (pb.hFileInfo.ioFlAttrib & 0x10) {
				if (getDirectories) {
					++*actItemCount; /* keep this item */
					++items; /* point to next item */
				}
			}
			else {
				if (getFiles) {
					++*actItemCount; /* keep this item */
					++items; /* point to next item */
				}
			}
		}
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	BumpDate(short vRefNum,
						 long dirID,
						 StringPtr name)
/* Given a file or directory, change its modification date to the current date/time. */
{
	CInfoPBRec pb;
	OSErr error;
	unsigned long secs;

	pb.hFileInfo.ioNamePtr = name;
	pb.hFileInfo.ioVRefNum = vRefNum;
	pb.hFileInfo.ioDirID = dirID;
	pb.hFileInfo.ioFDirIndex = 0;	/* use ioNamePtr and ioDirID */
	error = PBGetCatInfoSync(&pb);
	if (error == noErr)
	{
		GetDateTime(&secs);
		/* set mod date to current date, or one second into the future
			if mod date = current date */
		pb.hFileInfo.ioFlMdDat = (secs == pb.hFileInfo.ioFlMdDat) ? (++secs) : (secs);
		pb.hFileInfo.ioDirID = dirID;
		error = PBSetCatInfoSync(&pb);
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpBumpDate(const FSSpec *spec)
{
	return (BumpDate(spec->vRefNum, spec->parID, (StringPtr)spec->name));
}

/*****************************************************************************/

pascal	OSErr	ChangeCreatorType(short vRefNum,
								  long dirID,
								  const Str255 name,
								  OSType creator,
								  OSType fileType)
{
	CInfoPBRec pb;
	OSErr error;
	long parID;

	pb.hFileInfo.ioNamePtr = (StringPtr)name;
	pb.hFileInfo.ioVRefNum = vRefNum;
	pb.hFileInfo.ioDirID = dirID;
	pb.hFileInfo.ioFDirIndex = 0;	/* use ioNamePtr and ioDirID */
	error = PBGetCatInfoSync(&pb);
	if (error == noErr)
	{
		if (pb.hFileInfo.ioFlAttrib & 0x10)	/* if directory */
			return (notAFileErr);			/* do nothing and return error */
			
		parID = pb.hFileInfo.ioFlParID;	/* save parent dirID for BumpDate call */

		/* If creator not 0x00000000, change creator */
		if (creator != (OSType)0x00000000)
			pb.hFileInfo.ioFlFndrInfo.fdCreator = creator;
		
		/* If fileType not 0x00000000, change fileType */
		if (fileType != (OSType)0x00000000)
			pb.hFileInfo.ioFlFndrInfo.fdType = fileType;
			
		pb.hFileInfo.ioDirID = dirID;
		error = PBSetCatInfoSync(&pb);	/* now, save the new information back to disk */

		if (error == noErr)
			error = BumpDate(vRefNum, parID, nil);
				/* and bump the parent directory's mod date to wake up the Finder */
				/* to the change we just made */
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpChangeCreatorType(const FSSpec *spec,
									 OSType creator,
									 OSType fileType)
{
	return (ChangeCreatorType(spec->vRefNum, spec->parID, spec->name, creator, fileType));
}

/*****************************************************************************/

pascal	OSErr	ChangeFDFlags(short vRefNum,
							  long dirID,
							  StringPtr name,
							  Boolean	setBits,
							  unsigned short flagBits)
{
	CInfoPBRec pb;
	OSErr error;
	long parID;

	pb.hFileInfo.ioNamePtr = name;
	pb.hFileInfo.ioVRefNum = vRefNum;
	pb.hFileInfo.ioDirID = dirID;
	pb.hFileInfo.ioFDirIndex = 0;	/* use ioNamePtr and ioDirID */
	error = PBGetCatInfoSync(&pb);
	if (error == noErr)
	{
		parID = pb.hFileInfo.ioFlParID;	/* save parent dirID for BumpDate call */

		pb.hFileInfo.ioFlFndrInfo.fdFlags = (setBits) ? 
			(pb.hFileInfo.ioFlFndrInfo.fdFlags | flagBits) : /* OR in the bits */
			(pb.hFileInfo.ioFlFndrInfo.fdFlags & (0xffff ^ flagBits)); /* AND out the bits */
				/* set or clear the appropriate bits in the Finder flags */
			
		pb.hFileInfo.ioDirID = dirID;
		error = PBSetCatInfoSync(&pb);	/* now, save the new information back to disk */

		if (error == noErr)
			error = BumpDate(vRefNum, parID, nil);
				/* and bump the parent directory's mod date to wake up the Finder */
				/* to the change we just made */
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpChangeFDFlags(const FSSpec *spec,
								 Boolean setBits,
								 unsigned short flagBits)
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, setBits, flagBits));
}

/*****************************************************************************/

pascal	OSErr	SetIsInvisible(short vRefNum,
							   long dirID,
							   StringPtr name)
	/* Given a file or directory, make it invisible. */
{
	return (ChangeFDFlags(vRefNum, dirID, name, true, 0x4000));
}

/*****************************************************************************/

pascal	OSErr	FSpSetIsInvisible(const FSSpec *spec)
	/* Given a file or directory, make it invisible. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, true, 0x4000));
}

/*****************************************************************************/

pascal	OSErr	ClearIsInvisible(short vRefNum,
								 long dirID,
								 StringPtr name)
	/* Given a file or directory, make it visible. */
{
	return (ChangeFDFlags(vRefNum, dirID, name, false, 0x4000));
}

/*****************************************************************************/

pascal	OSErr	FSpClearIsInvisible(const FSSpec *spec)
	/* Given a file or directory, make it visible. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, false, 0x4000));
}

/*****************************************************************************/

pascal	OSErr	SetNameLocked(short vRefNum,
							  long dirID,
							  StringPtr name)
	/* Given a file or directory, lock its name. */
{
	return (ChangeFDFlags(vRefNum, dirID, name, true, 0x1000));
}

/*****************************************************************************/

pascal	OSErr	FSpSetNameLocked(const FSSpec *spec)
	/* Given a file or directory, lock its name. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, true, 0x1000));
}

/*****************************************************************************/

pascal	OSErr	ClearNameLocked(short vRefNum,
								long dirID,
								StringPtr name)
	/* Given a file or directory, unlock its name. */
{
	return (ChangeFDFlags(vRefNum, dirID, name, false, 0x1000));
}

/*****************************************************************************/

pascal	OSErr	FSpClearNameLocked(const FSSpec *spec)
	/* Given a file or directory, unlock its name. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, false, 0x1000));
}

/*****************************************************************************/

pascal	OSErr	SetIsStationery(short vRefNum,
								long dirID,
								const Str255 name)
	/* Given a file, make it a stationery pad. */
{
	return (ChangeFDFlags(vRefNum, dirID, (StringPtr)name, true, 0x0800));
}

/*****************************************************************************/

pascal	OSErr	FSpSetIsStationery(const FSSpec *spec)
	/* Given a file, make it a stationery pad. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, true, 0x0800));
}

/*****************************************************************************/

pascal	OSErr	ClearIsStationery(short vRefNum,
								  long dirID,
								  const Str255 name)
	/* Given a file, clear the stationery bit. */
{
	return (ChangeFDFlags(vRefNum, dirID, (StringPtr)name, false, 0x0800));
}

/*****************************************************************************/

pascal	OSErr	FSpClearIsStationery(const FSSpec *spec)
	/* Given a file, clear the stationery bit. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, false, 0x0800));
}

/*****************************************************************************/

pascal	OSErr	SetHasCustomIcon(short vRefNum,
								 long dirID,
								 StringPtr name)
	/* Given a file or directory, indicate that it has a custom icon. */
{
	return (ChangeFDFlags(vRefNum, dirID, name, true, 0x0400));
}

/*****************************************************************************/

pascal	OSErr	FSpSetHasCustomIcon(const FSSpec *spec)
	/* Given a file or directory, indicate that it has a custom icon. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, true, 0x0400));
}

/*****************************************************************************/

pascal	OSErr	ClearHasCustomIcon(short vRefNum,
								   long dirID,
								   StringPtr name)
	/* Given a file or directory, indicate that it does not have a custom icon. */
{
	return (ChangeFDFlags(vRefNum, dirID, name, false, 0x0400));
}

/*****************************************************************************/

pascal	OSErr	FSpClearHasCustomIcon(const FSSpec *spec)
	/* Given a file or directory, indicate that it does not have a custom icon. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, false, 0x0400));
}

/*****************************************************************************/

pascal	OSErr	ClearHasBeenInited(short vRefNum,
								   long dirID,
								   StringPtr name)
	/* Given a file, clear its "has been inited" bit. */
{
	return (ChangeFDFlags(vRefNum, dirID, (StringPtr)name, false, 0x0100));
}

/*****************************************************************************/

pascal	OSErr	FSpClearHasBeenInited(const FSSpec *spec)
	/* Given a file, clear its "has been inited" bit. */
{
	return (ChangeFDFlags(spec->vRefNum, spec->parID, (StringPtr)spec->name, false, 0x0100));
}

/*****************************************************************************/

pascal	OSErr	CopyFileMgrAttributes(short srcVRefNum,
									  long srcDirID,
									  StringPtr srcName,
									  short dstVRefNum,
									  long dstDirID,
									  StringPtr dstName,
									  Boolean copyLockBit)
{
	UniversalFMPB pb;
	OSErr error;
	Boolean objectIsDirectory;

	pb.ciPB.hFileInfo.ioVRefNum = srcVRefNum;
	pb.ciPB.hFileInfo.ioDirID = srcDirID;
	pb.ciPB.hFileInfo.ioNamePtr = srcName;
	pb.ciPB.hFileInfo.ioFDirIndex = 0;
	error = PBGetCatInfoSync(&pb.ciPB);
	if (error == noErr)
	{
		objectIsDirectory = (pb.ciPB.hFileInfo.ioFlAttrib & 0x10);
		pb.ciPB.hFileInfo.ioVRefNum = dstVRefNum;
		pb.ciPB.hFileInfo.ioDirID = dstDirID;
		pb.ciPB.hFileInfo.ioNamePtr = dstName;
		/* don't copy the hasBeenInited bit */
		pb.ciPB.hFileInfo.ioFlFndrInfo.fdFlags = (pb.ciPB.hFileInfo.ioFlFndrInfo.fdFlags & 0xfeff);
		error = PBSetCatInfoSync(&pb.ciPB);
		if ((error == noErr) && (copyLockBit) && (pb.ciPB.hFileInfo.ioFlAttrib & 0x01))
		{
			error = PBHSetFLockSync(&pb.hPB);
			if ((error != noErr) && (objectIsDirectory))
				error = noErr; /* ignore lock errors if destination is directory */
		}
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpCopyFileMgrAttributes(const FSSpec *srcSpec,
										 const FSSpec *dstSpec,
										 Boolean copyLockBit)
{
	return (CopyFileMgrAttributes(srcSpec->vRefNum, srcSpec->parID, (StringPtr)srcSpec->name,
								  dstSpec->vRefNum, dstSpec->parID, (StringPtr)dstSpec->name,
								  copyLockBit));
}

/*****************************************************************************/

pascal	OSErr	HOpenAware(short vRefNum,
						   long dirID,
						   const Str255 fileName,
						   short denyModes,
						   short *refNum)
{
	HParamBlockRec pb;
	OSErr error;

	pb.ioParam.ioVersNum = 0;
	pb.ioParam.ioMisc = nil;
	pb.fileParam.ioNamePtr = (StringPtr)fileName;
	pb.fileParam.ioVRefNum = vRefNum;
	pb.accessParam.ioDenyModes = denyModes;
	pb.fileParam.ioDirID = dirID;
	error = PBHOpenDenySync(&pb);				/* Try OpenDeny first */
	if (error == paramErr)
	{
		/* OpenDeny not supported, so try File Manager Open functions */
		/* Set File Manager permissions to closest thing possible */
		pb.ioParam.ioPermssn = ((denyModes == dmWr) || (denyModes == dmRdWr)) ? (fsRdWrShPerm) : (denyModes % 4);

		error = PBHOpenDFSync(&pb);				/* Try OpenDF */
		if (error == paramErr)
			error = PBHOpenSync(&pb);			/* OpenDF not supported, so try Open */
	}
	*refNum = pb.ioParam.ioRefNum;
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpOpenAware(const FSSpec *spec,
							 short denyModes,
							 short *refNum)
{
	return (HOpenAware(spec->vRefNum, spec->parID, spec->name, denyModes, refNum));
}

/*****************************************************************************/

pascal	OSErr	HOpenRFAware(short vRefNum,
							 long dirID,
							 const Str255 fileName,
							 short denyModes,
							 short *refNum)
{
	HParamBlockRec pb;
	OSErr error;

	pb.fileParam.ioNamePtr = (StringPtr)fileName;
	pb.fileParam.ioVRefNum = vRefNum;
	pb.ioParam.ioVersNum = 0;
	pb.accessParam.ioDenyModes = denyModes;
	pb.ioParam.ioMisc = nil;
	pb.fileParam.ioDirID = dirID;
	error = PBHOpenRFDenySync(&pb);				/* Try OpenRFDeny first */
	if (error == paramErr)
	{
		/* OpenRFDeny not supported, so try File Manager OpenRF function */
		/* Set File Manager permissions to closest thing possible */
		pb.ioParam.ioPermssn = ((denyModes == dmWr) || (denyModes == dmRdWr)) ? (fsRdWrShPerm) : (denyModes % 4);

		error = PBHOpenRFSync(&pb);				/* Try OpenRF */
	}
	*refNum = pb.ioParam.ioRefNum;
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpOpenRFAware(const FSSpec *spec,
							   short denyModes,
							   short *refNum)
{
	return (HOpenRFAware(spec->vRefNum, spec->parID, spec->name, denyModes, refNum));
}

/*****************************************************************************/

pascal	OSErr	FSReadNoCache(short refNum,
							  long *count,
							  void *buffPtr)
{
	ParamBlockRec pb;
	OSErr error;

	pb.ioParam.ioRefNum = refNum;
	pb.ioParam.ioBuffer = buffPtr;
	pb.ioParam.ioReqCount = *count;
	pb.ioParam.ioPosMode = fsAtMark + 0x0020;	/* fsAtMark + noCacheBit */
	pb.ioParam.ioPosOffset = 0;
	error = PBReadSync(&pb);
	*count = pb.ioParam.ioActCount;
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSWriteNoCache(short refNum,
							   long *count,
							   const void *buffPtr)
{
	ParamBlockRec pb;
	OSErr error;

	pb.ioParam.ioRefNum = refNum;
	pb.ioParam.ioBuffer = (Ptr)buffPtr;
	pb.ioParam.ioReqCount = *count;
	pb.ioParam.ioPosMode = fsAtMark + 0x0020;	/* fsAtMark + noCacheBit */
	pb.ioParam.ioPosOffset = 0;
	error = PBWriteSync(&pb);
	*count = pb.ioParam.ioActCount;
	return (error);
}

/*****************************************************************************/

pascal	OSErr	CopyFork(short srcRefNum,
						 short dstRefNum,
						 void *copyBufferPtr,
						 long copyBufferSize)
{
	ParamBlockRec srcPB;
	ParamBlockRec dstPB;
	OSErr srcError;
	OSErr dstError;

	if ((copyBufferPtr == nil) || (copyBufferSize == 0))
		return (paramErr);
	
	srcPB.ioParam.ioRefNum = srcRefNum;
	dstPB.ioParam.ioRefNum = dstRefNum;

	/* preallocate the destination fork and */
	/* ensure the destination fork's EOF is correct after the copy */
	srcError = PBGetEOFSync(&srcPB);
	if (srcError != noErr)
		return (srcError);
	dstPB.ioParam.ioMisc = srcPB.ioParam.ioMisc;
	dstError = PBSetEOFSync(&dstPB);
	if (dstError != noErr)
		return (dstError);

	/* reset source fork's mark */
	srcPB.ioParam.ioPosMode = fsFromStart;
	srcPB.ioParam.ioPosOffset = 0;
	srcError = PBSetFPosSync(&srcPB);
	if (srcError != noErr)
		return (srcError);

	/* reset destination fork's mark */
	dstPB.ioParam.ioPosMode = fsFromStart;
	dstPB.ioParam.ioPosOffset = 0;
	dstError = PBSetFPosSync(&srcPB);
	if (dstError != noErr)
		return (dstError);

	/* set up fields that won't change in the loop */
	srcPB.ioParam.ioBuffer = copyBufferPtr;
	srcPB.ioParam.ioPosMode = fsAtMark + 0x0020;/* fsAtMark + noCacheBit */
	srcPB.ioParam.ioReqCount = ((copyBufferSize >= 512) && (copyBufferSize % 512)) ? (copyBufferSize / 512) * 512 : (copyBufferSize);
	/* If copyBufferSize is greater than 512 bytes, make it a multiple of 512 bytes */
	/* This will make writes on local volumes faster */

	dstPB.ioParam.ioBuffer = copyBufferPtr;
	dstPB.ioParam.ioPosMode = fsAtMark + 0x0020;/* fsAtMark + noCacheBit */

	while ((srcError == noErr) && (dstError == noErr))
	{
		srcError = PBReadSync(&srcPB);
		dstPB.ioParam.ioReqCount = srcPB.ioParam.ioActCount;
		dstError = PBWriteSync(&dstPB);
	}

	/* make sure there were no errors at the destination */
	if (dstError != noErr)
		return (dstError);

	/* make sure the only error at the source was eofErr */
	if (srcError != eofErr)
		return (srcError);

	return (noErr);
}

/*****************************************************************************/

pascal	OSErr	GetFileLocation(short refNum,
								short *vRefNum,
								long *dirID,
								StringPtr fileName)
{
	FCBPBRec pb;
	OSErr error;

	pb.ioNamePtr = fileName;
	pb.ioVRefNum = 0;
	pb.ioRefNum = refNum;
	pb.ioFCBIndx = 0;
	error = PBGetFCBInfoSync(&pb);
	*vRefNum = pb.ioFCBVRefNum;
	*dirID = pb.ioFCBParID;
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpGetFileLocation(short refNum,
								   FSSpec *spec)
{
	return (GetFileLocation(refNum, &(spec->vRefNum), &(spec->parID), spec->name));
}

/*****************************************************************************/

pascal	OSErr	CopyDirectoryAccess(short srcVRefNum,
									long srcDirID,
									StringPtr srcName,
									short dstVRefNum,
									long dstDirID,
									StringPtr dstName)
{	
	OSErr err;
	GetVolParmsInfoBuffer infoBuffer;	/* Where PBGetVolParms dumps its info */
	long	dstServerAdr;				/* AppleTalk server address of destination (if any) */
	Boolean	dstHasBlankAccessPrivileges;
	long	ownerID, groupID, accessRights;
	long	tempLong;

	/* See if destination supports directory access control */
	tempLong = sizeof(infoBuffer);
	err = HGetVolParms(dstName, dstVRefNum, &infoBuffer, &tempLong);
	if ((err != noErr) && (err != paramErr))
		return (err);
	if ((err != paramErr) && hasAccessCntl(infoBuffer)) {
		dstServerAdr = infoBuffer.vMServerAdr;
		dstHasBlankAccessPrivileges = hasBlankAccessPrivileges(infoBuffer);
	}
	else
		/* If destination doesn't support access privileges, */
		/* then there's nothing left to do here */
		return (noErr);

	/* We may have to do something with access privileges at the destination. */
	
	accessRights = 0;	/* clear so we can tell if anything was done with this */

	/* See if source supports directory access control and is on same server */
	tempLong = sizeof(infoBuffer);
	err = HGetVolParms(srcName, srcVRefNum, &infoBuffer, &tempLong);
	if ((err != noErr) && (err != paramErr))
		return (err);
	if ((err != paramErr) && hasAccessCntl(infoBuffer)) {
		/* Make sure both locations are on the same file server */
		if (dstServerAdr == infoBuffer.vMServerAdr) {
			/* copy'm */
			err = HGetDirAccess(srcVRefNum, srcDirID, srcName, &ownerID, &groupID, &accessRights);
			if (err == noErr)
				err = HSetDirAccess(dstVRefNum, dstDirID, dstName, ownerID, groupID, accessRights);
		}
	}
	return (err);
}

/*****************************************************************************/

pascal	OSErr	FSpCopyDirectoryAccess(const FSSpec *srcSpec,
									   const FSSpec *dstSpec)
{
	return (CopyDirectoryAccess(srcSpec->vRefNum, srcSpec->parID, (StringPtr)srcSpec->name,
								dstSpec->vRefNum, dstSpec->parID, (StringPtr)dstSpec->name));
}

/*****************************************************************************/

pascal	void	BuildAFPVolMountInfo(short theFlags,
									 char theNBPInterval,
									 char theNBPCount,
									 short theUAMType,
									 Str31 theZoneName,
									 Str31 theServerName,
									 Str27 theVolName,
									 Str31 theUserName,
									 Str8 theUserPassWord,
									 Str8 theVolPassWord,
									 MyAFPVolMountInfoPtr theAFPInfo)
{
	/* Fill in an AFPVolMountInfo record that can be passed to VolumeMount */
	theAFPInfo->length = sizeof(MyAFPVolMountInfo);
	theAFPInfo->media = AppleShareMediaType;
	theAFPInfo->flags = theFlags;
	theAFPInfo->nbpInterval = theNBPInterval;
	theAFPInfo->nbpCount = theNBPCount;
	theAFPInfo->uamType = theUAMType;
	theAFPInfo->zoneNameOffset = (long)theAFPInfo->zoneName - (long)theAFPInfo;
	theAFPInfo->serverNameOffset = (long)theAFPInfo->serverName - (long)theAFPInfo;
	theAFPInfo->volNameOffset = (long)theAFPInfo->volName - (long)theAFPInfo;
	theAFPInfo->userNameOffset = (long)theAFPInfo->userName - (long)theAFPInfo;
	theAFPInfo->userPasswordOffset = (long)theAFPInfo->userPassword - (long)theAFPInfo;
	theAFPInfo->volPasswordOffset = (long)theAFPInfo->volPassword - (long)theAFPInfo;
	pcpy(theAFPInfo->zoneName, theZoneName);
	pcpy(theAFPInfo->serverName, theServerName);
	pcpy(theAFPInfo->volName, theVolName);
	pcpy(theAFPInfo->userName, theUserName);
	pcpy(theAFPInfo->userPassword, theUserPassWord);
	pcpy(theAFPInfo->volPassword, theVolPassWord);
}

/*****************************************************************************/

pascal	OSErr	RetrieveAFPVolMountInfo(AFPVolMountInfoPtr theAFPInfo,
										short *theFlags,
										short *theUAMType,
										StringPtr theZoneName,
										StringPtr theServerName,
										StringPtr theVolName,
										StringPtr theUserName)
{
	/* Retrieve the AFP mounting information from an AFPVolMountInfo record. */
	if (theAFPInfo->media != AppleShareMediaType)
		return (paramErr);
	
	*theFlags = theAFPInfo->flags;
	*theUAMType = theAFPInfo->uamType;
	pcpy(theZoneName, (StringPtr)((long)theAFPInfo + theAFPInfo->zoneNameOffset));
	pcpy(theServerName, (StringPtr)((long)theAFPInfo + theAFPInfo->serverNameOffset));
	pcpy(theVolName, (StringPtr)((long)theAFPInfo + theAFPInfo->volNameOffset));
	pcpy(theUserName, (StringPtr)((long)theAFPInfo + theAFPInfo->userNameOffset));
}

/*****************************************************************************/

pascal	OSErr	GetUGEntries(short objType,
							 UGEntryPtr entries,
							 long reqEntryCount,
							 long *actEntryCount,
							 long *objID)
{
	HParamBlockRec pb;
	OSErr error;
	UGEntry *endEntryArray = entries + reqEntryCount;

	pb.objParam.ioObjType = objType;
	*actEntryCount = 0;
	for (; (entries < endEntryArray) && (error == noErr); ++entries)
	{
		pb.objParam.ioObjNamePtr = (StringPtr)entries->name;
		pb.objParam.ioObjID = *objID;
		error = PBGetUGEntrySync(&pb);
		if (error == noErr)
		{
			entries->objID = *objID = pb.objParam.ioObjID;
			entries->objType = objType;
			++*actEntryCount;
		}
	}
	return (error);
}

/*****************************************************************************/

/*	Desktop Manager calls	*/

/*****************************************************************************/

pascal	OSErr	CopyComment(short srcVRefNum,
							long srcDirID,
							StringPtr srcName,
							short dstVRefNum,
							long dstDirID,
							StringPtr dstName)
/* Both volumes must support the Desktop Manager for this to work */
{
	DTPBRec pb;
	OSErr error;
	unsigned char commentBuffer[199];
	long commentLength;

	pb.ioNamePtr = srcName;
	pb.ioVRefNum = srcVRefNum;
	error = PBDTOpenInform(&pb);
	if (error == paramErr)
		error = PBDTGetPath(&pb);
	if (error == noErr)
	{
		pb.ioNamePtr = srcName;
		pb.ioDirID = srcDirID;
		pb.ioDTBuffer = (Ptr) & commentBuffer;
		error = PBDTGetCommentSync(&pb);
		if (error == noErr)
		{
			commentLength = pb.ioDTActCount;
			pb.ioNamePtr = dstName;
			pb.ioVRefNum = dstVRefNum;
			error = PBDTOpenInform(&pb);
			if (error == paramErr)
				error = PBDTGetPath(&pb);
			if (error == noErr)
			{
				pb.ioNamePtr = dstName;
				pb.ioDirID = dstDirID;
				pb.ioDTBuffer = (Ptr) & commentBuffer;
				pb.ioDTReqCount = commentLength;
				error = PBDTSetCommentSync(&pb);
			}
		}
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpCopyComment(const FSSpec *srcSpec,
							   const FSSpec *dstSpec)
/* Both volumes must support the Desktop Manager for this to work */
{
	return (CopyComment(srcSpec->vRefNum, srcSpec->parID, (StringPtr)srcSpec->name,
						dstSpec->vRefNum, dstSpec->parID, (StringPtr)dstSpec->name));
}

/*****************************************************************************/

pascal	OSErr	SetComment(short vRefNum,
							 long dirID,
							 StringPtr name,
							 const Str255 comment)
{
	DTPBRec pb;
	OSErr error;

	pb.ioNamePtr = name;
	pb.ioVRefNum = vRefNum;
	error = PBDTOpenInform(&pb);
	if (error == paramErr)
		error = PBDTGetPath(&pb);
	if (error == noErr)
	{
		pb.ioNamePtr = name;
		pb.ioDirID = dirID;
		pb.ioDTBuffer = (Ptr)&comment[1];
		pb.ioDTReqCount = comment[0];
		error = PBDTSetCommentSync(&pb);
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpSetComment(const FSSpec *spec,
							  const Str255 comment)
{
	return (SetComment(spec->vRefNum, spec->parID, (StringPtr)spec->name, comment));
}

/*****************************************************************************/

pascal	OSErr	GetComment(short vRefNum,
						   long dirID,
						   StringPtr name,
						   StringPtr comment)
{
	DTPBRec pb;
	OSErr error;

	if (comment == nil)
		return (paramErr);
	
	pb.ioNamePtr = name;
	pb.ioVRefNum = vRefNum;
	error = PBDTOpenInform(&pb);
	if (error == paramErr)
		error = PBDTGetPath(&pb);
	if (error == noErr)
	{
		pb.ioNamePtr = name;
		pb.ioDirID = dirID;
		pb.ioDTBuffer = (Ptr)&comment[1];
		error = PBDTGetCommentSync(&pb);
		if (error == noErr)
			comment[0] = pb.ioDTActCount;
		else
			comment[0] = 0;
	}
	return (error);
}

/*****************************************************************************/

pascal	OSErr	FSpGetComment(const FSSpec *spec,
							  StringPtr comment)
{
	return (GetComment(spec->vRefNum, spec->parID, (StringPtr)spec->name, comment));
}

/*****************************************************************************/

